#!/usr/bin/env gnuplot
#
# Author:  Carlos E. Budde
# Date:    06.10.2019
# License: GPLv3
#

## Check external variables were defined
if (!exists("DIR"))   exit error "[ERROR] 'DIR' undefined"
if (!exists("FLAT"))  exit error "[ERROR] 'FLAT' undefined"
if (!exists("RES_0")) exit error "[ERROR] 'RES_0' undefined"
if (!exists("RES_1")) exit error "[ERROR] 'RES_1' undefined"
if (!exists("RES_2")) exit error "[ERROR] 'RES_2' undefined"
if (!exists("RES_3")) exit error "[ERROR] 'RES_3' undefined"
if (!exists("RES_4")) exit error "[ERROR] 'RES_4' undefined"
if (!exists("RES_5")) exit error "[ERROR] 'RES_5' undefined"

NUM_RUNS=6
NUM_MODELS=3
YMIN=7e-8
YMAX=4e-5

RES_0_T=RES_0[1:3]." {".RES_0[5:]."}"
RES_1_T=RES_1[1:3]." {".RES_1[5:]."}"
RES_2_T=RES_2[1:3]." {".RES_2[5:]."}"
RES_3_T=RES_3[1:3]." {".RES_3[5:]."}"
RES_4_T=RES_4[1:3]." {".RES_4[5:]."}"
RES_5_T=RES_5[1:3]." {".RES_5[5:]."}"

# Output file setup
load "palettes/inferno.pal"  # color palette: palettes/{viridis,plasma,inferno}.pal
set terminal qt size 1200,266 persist
###	## set encoding utf8
###	## set term pdf dashed enhanced size 7,6 font 'Verdana,15'
# set terminal pdf enhanced size 8,3
# set out DIR."/plot.pdf"
#
unset title
set style data histogram
set style histogram errorbars gap 1.1
set style fill solid 0.8 noborder
set log y
set format y "%.0e"  # scientific notation: 1e-04
set ytics font ",20"
set xtics font ",20" nomirror offset 0,-.7
set grid ytics lw 1.2 #dt 3 
set yrange [YMIN:YMAX]
set xrange [-1:NUM_MODELS]
set offsets -0.3,-0.3,0,0
set lmargin 14
set bmargin 5
#set xlabel "Models"       font ",12" offset 0,-.5
#set ylabel "CI precision" font ",12" offset -3.5
set key font ",16" samplen 2
set key out right
set bars .7 lc -1 lw 1.5
#
plot \
	DIR."/precision_FLAT.dat"       u 3:4:xtic(2) ls 1 t "CMC", \
		"" u ($1-.375):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_0.".dat"  u 3:4:xtic(2) ls 4 t RES_0_T, \
		"" u ($1-.250):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_1.".dat"  u 3:4:xtic(2) ls 6 t RES_1_T, \
		"" u ($1-.130):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_2.".dat"  u 3:4:xtic(2) ls 2 t RES_2_T, \
		"" u ($1+.000):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_3.".dat"  u 3:4:xtic(2) ls 5 t RES_3_T, \
		"" u ($1+.130):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_4.".dat"  u 3:4:xtic(2) ls 7 t RES_4_T, \
		"" u ($1+.250):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".RES_5.".dat"  u 3:4:xtic(2) ls 3 t RES_5_T, \
		"" u ($1+.375):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \

# Lines "" u ($1+.390):(1.5*YMIN):5 w labels ... place the white CI counts
#              ^^^^^^^ this is the horizontal alignment, to match the proper bar
# For
# 9 RUN types, use horizontal alignemnt values:
#   -.395  -.300  -.200  -.095  +.000  +.095  +.200  +.300  +.395 
# 8 RUN types:
#   -.390  -.281  -.167  -.055  +.055  +.167  +.281  +.390
# 7 RUN types:
#   -.375  -.250  -.130  +.000  +.130  +.250  +.375
# 6 RUN types:
#   -.358  -.213  -.073  +.073  +.213  +.358
# 5 RUN types:
#   -.333  -.166  +.000  +.166  +.333

pause -1
