#!/usr/bin/env gnuplot
#
# Author:  Carlos E. Budde
# Date:    06.10.2019
# License: GPLv3
#

## Check external variables were defined
if (!exists("DIR"))    exit error "[ERROR] 'DIR' undefined"
if (!exists("AH"))     exit error "[ERROR] 'AH' undefined"
if (!exists("AUTO_0")) exit error "[ERROR] 'AUTO_0' undefined"
if (!exists("AUTO_1")) exit error "[ERROR] 'AUTO_1' undefined"
if (!exists("AUTO_2")) exit error "[ERROR] 'AUTO_2' undefined"
if (!exists("AUTO_3")) exit error "[ERROR] 'AUTO_3' undefined"
if (!exists("AUTO_4")) exit error "[ERROR] 'AUTO_4' undefined"

NUM_RUNS=6
NUM_MODELS=3
YMIN=7.0e-16

AUTO_0_T=AUTO_0[1:4]." {".AUTO_0[6:]."}"
AUTO_1_T=AUTO_1[1:4]." {".AUTO_1[6:]."}"
AUTO_2_T=AUTO_2[1:4]." {".AUTO_2[6:]."}"
AUTO_3_T=AUTO_3[1:4]." {".AUTO_3[6:]."}"
AUTO_4_T=AUTO_4[1:4]." {".AUTO_4[6:]."}"

# Output file setup
load "palettes/viridis.pal"  # color palette: palettes/{viridis,plasma,inferno}.pal
set terminal qt size 1200,250 persist
###	## set encoding utf8
###	## set term pdf dashed enhanced size 7,6 font 'Verdana,15'
# set terminal pdf enhanced size 8,3
# set out DIR."/plot.pdf"
#
unset title
set style data histogram
set style histogram errorbars gap 1.1
set style fill solid 0.8 noborder
set log y
set format y "%.0e"  # scientific notation: 1e-04
set ytics font ",20"
set xtics font ",18" nomirror offset 0,-.7
set grid ytics lw 1.2 #dt 3 
set yrange [YMIN:*]
set xrange [-1:NUM_MODELS]
set offsets -0.3,-0.3,0,0
set lmargin 14
set bmargin 5
#set xlabel "Models"       font ",12" offset 0,-.5
#set ylabel "CI precision" font ",12" offset -3.5
set key font ",16" samplen 2
set key out right
set bars .7 lc -1 lw 1.5
#
plot \
	DIR."/precision_AH.dat"         u 3:4:xtic(2) ls 1 t "AD HOC", \
		"" u ($1-.358):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".AUTO_0.".dat" u 3:4:xtic(2) ls 5 t AUTO_0_T, \
		"" u ($1-.213):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".AUTO_1.".dat" u 3:4:xtic(2) ls 2 t AUTO_1_T, \
		"" u ($1-.073):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".AUTO_2.".dat" u 3:4:xtic(2) ls 7 t AUTO_2_T, \
		"" u ($1+.073):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".AUTO_3.".dat" u 3:4:xtic(2) ls 4 t AUTO_3_T, \
		"" u ($1+.213):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \
	DIR."/precision_".AUTO_4.".dat" u 3:4:xtic(2) ls 8 t AUTO_4_T, \
		"" u ($1+.358):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "black" t "", \

#	"precision_XXXXX_RES_HYB8.dat"  u 3:4:xtic(2) ls 2 t "RES-8", \
#		"" u ($1+.333):(1.5*YMIN):5 w labels font ":bold,18" tc rgb "white" t "", \

# Lines "" u ($1+.390):(1.5*YMIN):5 w labels ... place the white CI counts
#              ^^^^^^^ this is the horizontal alignment, to match the proper bar
# For
# 9 RUN types, use horizontal alignemnt values:
#   -.395  -.300  -.200  -.095  +.000  +.095  +.200  +.300  +.395 
# 8 RUN types:
#   -.390  -.281  -.167  -.055  +.055  +.167  +.281  +.390
# 7 RUN types:
#   -.375  -.250  -.130  +.000  +.130  +.250  +.375
# 6 RUN types:
#   -.358  -.213  -.073  +.073  +.213  +.358
# 5 RUN types:
#   -.333  -.166  +.000  +.166  +.333

pause -1
